/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_api.cpp
 *
 * Purpose:     Main (platform-independent) implementation file for the recls API.
 *
 * Created:     16th August 2003
 * Updated:     9th September 2003
 *
 * Copyright:   Synesis Software Pty Ltd, (c) 2003. All rights reserved.
 *
 * Home:        http://www.synesis.com.au/software
 *              http://recls.org/
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#define RECLS_PURE_API

#include "recls.h"
#include "recls_internal.h"
#include "recls_assert.h"

#include <stlsoft_nulldef.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Search enumeration
 */

RECLS_FNDECL(recls_rc_t) Recls_GetDetails(  hrecls_t        hSrch
                                        ,   recls_info_t    *pinfo)
{
    ReclsSearchInfo     *si     =   ReclsSearchInfo::FromHandle(hSrch);

    recls_assert(NULL != si);
    recls_assert(NULL != pinfo);

    return si->GetDetails(pinfo);
}

/* /////////////////////////////////////////////////////////////////////////////
 * Error handling
 */

RECLS_FNDECL(recls_rc_t) Recls_GetLastError(hrecls_t hSrch)
{
    ReclsSearchInfo *si =   ReclsSearchInfo::FromHandle(hSrch);

    recls_assert(NULL != si);

    return si->GetLastError();
}

RECLS_FNDECL(int) Recls_GetLastErrorString( hrecls_t    hSrch
                                        ,   char        *buffer
                                        ,   size_t      cchBuffer)
{
    return Recls_GetErrorString(Recls_GetLastError(hSrch), buffer, cchBuffer);
}

/* /////////////////////////////////////////////////////////////////////////////
 * Property elicitation
 */

RECLS_FNDECL(size_t) Recls_GetPathProperty( recls_info_t    fileInfo
                                        ,   char            *buffer
                                        ,   size_t          cchBuffer)
{
    recls_assert(NULL != fileInfo);

    return Recls_GetStringProperty_(&fileInfo->path, buffer, cchBuffer);
}

RECLS_FNDECL(size_t) Recls_GetDirectoryProperty(    recls_info_t    fileInfo
                                                ,   char            *buffer
                                                ,   size_t          cchBuffer)
{
    recls_assert(NULL != fileInfo);

    return Recls_GetStringProperty_(&fileInfo->directory, buffer, cchBuffer);
}

RECLS_FNDECL(size_t) Recls_GetFileProperty( recls_info_t    fileInfo
                                        ,   char            *buffer
                                        ,   size_t          cchBuffer)
{
    recls_assert(NULL != fileInfo);

    struct recls_strptrs_t file =
    {
            fileInfo->fileName.begin /* File is defined by start of fileName ... */
        ,   fileInfo->fileExt.end    /* ... to end of fileExt. */
    };

    return Recls_GetStringProperty_(&file, buffer, cchBuffer);
}

RECLS_FNDECL(size_t) Recls_GetFileNameProperty( recls_info_t    fileInfo
                                            ,   char            *buffer
                                            ,   size_t          cchBuffer)
{
    recls_assert(NULL != fileInfo);

    return Recls_GetStringProperty_(&fileInfo->fileName, buffer, cchBuffer);
}

RECLS_FNDECL(size_t) Recls_GetFileExtProperty(  recls_info_t    fileInfo
                                            ,   char            *buffer
                                            ,   size_t          cchBuffer)
{
    recls_assert(NULL != fileInfo);

    return Recls_GetStringProperty_(&fileInfo->fileExt, buffer, cchBuffer);
}

RECLS_FNDECL(size_t) Recls_GetDirectoryPartProperty(    recls_info_t    fileInfo
                                                    ,   int             part
                                                    ,   char            *buffer
                                                    ,   size_t          cchBuffer)
{
    recls_assert(NULL != fileInfo);

    size_t  cParts = fileInfo->directoryParts.end - fileInfo->directoryParts.begin;

    /* If they've not been created in the search, then they'll just */

    if(part < 0)
    {
        return cParts;
    }
    else
    {
        recls_assert(static_cast<size_t>(part) < cParts);

        return Recls_GetStringProperty_(&fileInfo->directoryParts.begin[part], buffer, cchBuffer);
    }
}

RECLS_FNDECL(recls_time_t) Recls_GetModificationTime(recls_info_t fileInfo)
{
    recls_assert(NULL != fileInfo);

    return fileInfo->modificationTime;
}

RECLS_FNDECL(recls_time_t) Recls_GetLastAccessTime(recls_info_t fileInfo)
{
    recls_assert(NULL != fileInfo);

    return fileInfo->lastAccessTime;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
