///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    DynSystem.h
// Package:     SDESolver
// Class:       CDynSystem
// Purpose:     Represents an Ito vector stochastic differential equation:
//                  dX = a(X,t)*dt + b(X,t)*dW
//              where a is the drift vector and b is the diffusion matrix
// Written:     Dave Burke      03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef DYNSYSTEM_H
#define DYNSYSTEM_H

class CDynSystem
{
    public:

    CDynSystem(void);
    virtual ~CDynSystem(void);

    // Override this method for the defined system
    virtual void GetDriftAndDiffusion(float nTime, float* pXin,
                                      float* pAout, float** ppBout) = 0;

    // Read only
    unsigned int GetDimension(void) { return m_nDimension; }

    protected:

    unsigned int m_nDimension;	// dimension of system
};

// Used for hooks into shared libraries
typedef CDynSystem* (*Create_t)(void);
typedef bool (*Destroy_t)(CDynSystem*);

#endif // DYNSYSTEM_H
