///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    EllipseDynSystem.h
// Package:     SDESolver
// Class:       CDynSystem
// Purpose:     SDE system in Ito form:
//                  dX1 = [dH/dX2 - l*(H)*dH/dX1]dt + [-l*sigma*dH/dX1]dW
//                  dX2 = [-dH/dX1 - l*(H)*dH/dX2]dt + [-l*sigma*dH/dX2]dW              ...1
//
//              where W is Wiener process and formally dW/dt = n(t) white noise, and
//              H is the equation for the Ellipse:
//                  X1^2/a^2 + X2^2/b^2 - 1                                             ...2
//              and l > 0, sigma is noise intensity
//
// Written:     Dave Burke      03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef ELLIPSEDYNSYSTEM_H
#define ELLIPSEDYNSYSTEM_H

#include "DynSystem.h"

class CEllipseDynSystem : public CDynSystem
{
    public:

    CEllipseDynSystem(void);

    virtual void GetDriftAndDiffusion(float nTime, float* pXin,
                                      float* pAout, float** ppBout);

    float m_nL;         // l from equation (1) above
	float m_nS;         // sigma from equation (1) above
	float m_nA;         // a from equation (2) above
	float m_nB;         // b from equation (2) above
};

// Factory methods to be used external to shared object
extern "C" CDynSystem* Create(void);
extern "C" void Destroy(CDynSystem* pDynSystem);

#endif // ELLIPSEDYNSYSTEM_H
