///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    GBMDynSystem.h
// Package:     SDESolver
// Class:       CDynSystem
// Purpose:     SDE system in Ito form:
//                  dX = mu*X*dt + sigma*X*dW                                           ...1
//              where W is Wiener process and formally dW/dt = n(t) is white noise.
//              This is Geometric Brownian Motion and has explicit solution of
//              X = X(0)exp((mu-0.5*sigma^2)*(t+sigma*W))
//
// Written:        Dave Burke        03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef GBMDYNSYSTEM_H
#define GBMDYNSYSTEM_H

#include "DynSystem.h"

class CGBMDynSystem : public CDynSystem
{
    public:

    CGBMDynSystem(void);

    virtual void GetDriftAndDiffusion(float nTime, float* pXin,
                                      float* pAout, float** ppBout);

    float m_nMu;            // mu from equation (1) above
    float m_nSigma;        // sigma from equation (1) above
};

// Factory methods to be used external to shared object
extern "C" CDynSystem* Create(void);
extern "C" void Destroy(CDynSystem* pDynSystem);

#endif // GBMDYNSYSTEM_H
