///////////////////////////////////////////////////////////////////////////////
//    Charles River Analytics Inc.
//    Copyright 2002-2003. All Rights Reserved.
//
//    ORIGINAL SYSTEM: The Vision Kit Library
//    SUBSYSTEM: Image
//    $Workfile: GammaCorrection.h $
//    $Revision: 2 $
//    Author(s): V.Ablavsky, J.Pollak, M.Stevens
//
//    Description
//    ===========
//
// Implements the gamma correction algorithms
//
///////////////////////////////////////////////////////////////////////////////

#include "Image.h"
#include <vector>
#include <fstream>
#include <cmath>

#ifndef GammaCorrection_h
#define GammaCorrection_h

namespace cuj
{

  // nested for loops, specialized for byte due to lut
  template<typename _Tp>  class GammaCorrection1;
  template<> class GammaCorrection1<byte>
  {

    public:

    // construct the lookup table
    GammaCorrection1(float gamma) : lut_(256)
    {
      for (size_t i = 0; i < 256; ++i)
      {
        const float original  = static_cast<float>(i) / 255.0;
        const float corrected =  pow(original, 1.0 / gamma);
        lut_[i] = static_cast<byte>(corrected * 255.0);
      }
    }
    
    // Process
    inline void operator()(const Image<byte> &in, Image<byte> &out)
    {
      for (size_t y = 0; y < in.height(); ++y)
      {
        for (size_t x = 0; x < in.width(); ++x)
        {
          out.set(x, y) = lut_[in.get(x, y)];
        }
      }
    }    
    
    private:

    // lookup table for values
    std::vector<byte> lut_;

  };

  // nested for loops, specialized for byte due to lut
  template<typename _Tp>  class GammaCorrection2;
  template<> class GammaCorrection2<byte>
  {

    public:

    // construct the lookup table    
    GammaCorrection2(float gamma) : lut_(256)
    {
      for (size_t i = 0; i < 256; ++i)
      {
        const float original  = static_cast<float>(i) / 255.0;
        const float corrected =  pow(original, 1.0 / gamma);
        lut_[i] = static_cast<byte>(corrected * 255.0);
      }
    }
    
    // Process
    template<typename _Tp>inline byte operator()(const _Tp &val)
    {
      return lut_[val];
    }    
    
    private:

    // lookup table for values
    std::vector<byte> lut_;

  };
}

#endif
