///////////////////////////////////////////////////////////////////////////////
//    Charles River Analytics Inc.
//    Copyright 2002-2003. All Rights Reserved.
//
//    ORIGINAL SYSTEM: The Vision Kit Library
//    SUBSYSTEM: Image
//    $Workfile: Image.h $
//    $Revision: 1 $
//    Author(s): V.Ablavsky, J.Pollak, M.Stevens
//
//    Description
//    ===========
//
// Naive ImageProcessing 101 Image representation
//
// Provides an incomplete image container definition based on
// gcc-3.2.0 container concepts
//
///////////////////////////////////////////////////////////////////////////////

#include <cstddef>
#include <vector>

#ifndef Image_h
#define Image_h

namespace cuj
{

  template<typename _Tp> class Image
  {
    
    // stl defines
    typedef _Tp 		value_type;
    typedef value_type* 	pointer;
    typedef const value_type* 	const_pointer;
    typedef pointer 	        iterator;
    typedef const_pointer       const_iterator;
    typedef value_type& 	reference;
    typedef const value_type& 	const_reference;
    typedef size_t 		size_type;
    typedef ptrdiff_t 		difference_type;

    public:

    // construct with known data elements
    Image(size_type width, size_type height) :
      _M_data(width * height),
      _M_width(width),
      _M_height(height)
    {
    }

    // access image dimensions
    inline size_type width() const
    {
      return _M_width;
    }
    inline size_type height() const
    {
      return _M_height;
    }

    // getter/setter methods
    inline value_type get(size_t x, size_t y) const
    {
      return _M_data[y * _M_width + x];
    }
    inline reference set(size_t x, size_t y)
    {
      return _M_data[y * _M_width + x];
    }

    // access iterators
    iterator begin()
    {
      return &(_M_data[0]);
    }
    iterator end()
    {
      return &(_M_data[_M_data.size()]);
    }
    const_iterator begin() const
    {
      return &(_M_data[0]);
    }
    const_iterator end() const
    {
      return &(_M_data[_M_data.size()]);
    }

    private:
    
    // data stored as a vector to allow compiler provided copy
    // construction, copying and destruction
    std::vector<value_type> _M_data;

    // image dimensions
    size_type _M_width;
    size_type _M_height;

  };
}

#endif
