///////////////////////////////////////////////////////////////////////////////
//    Charles River Analytics Inc.
//    Copyright 2002-2003. All Rights Reserved.
//
//    ORIGINAL SYSTEM: The Vision Kit Library
//    SUBSYSTEM: Image
//    $Workfile: ImageDefs.h $
//    $Revision: 1 $
//    Author(s): V.Ablavsky, J.Pollak, M.Stevens
//
//    Description
//    ===========
//
// Image type definitions
//
///////////////////////////////////////////////////////////////////////////////

#ifndef ImageDefs_h
#define ImageDefs_h

#include <iterator>

namespace cuj {
  
  typedef unsigned char byte;

  // some operations require that computation
  // be performed on a different type due to 
  // over/underflow
  template<typename _Tp> struct image_traits {
    typedef _Tp intermediate_type;
  };
  template<> struct image_traits<byte>{
    typedef int intermediate_type;
  };

  // A heavyweight forward iterator is a forward iterator
  // that is capable of providing light-weight proxies
  struct heavy_forward_iterator_tag
    : public std::forward_iterator_tag {
  };
}

#if defined(WIN32) && defined(_MSC_VER)

namespace std {
  template<>
    struct iterator_traits<cuj::byte*> {
    typedef ptrdiff_t difference_type;
    typedef cuj::byte value_type;
    typedef cuj::byte* pointer;
    typedef cuj::byte& reference;
    typedef random_access_iterator_tag iterator_category;
  };
  template<>
    struct iterator_traits<float*> {
    typedef ptrdiff_t difference_type;
    typedef float value_type;
    typedef float* pointer;
    typedef float& reference;
    typedef random_access_iterator_tag iterator_category;
  };
}

#endif


#endif
