///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    MilsteinSolver.h
// Package:     SDESolver
// Class:       CMilsteinSolver
// Purpose:     Milstein SDE solver. For simplicity we assume diagonal noise i.e.
//              bjk==0 for j!=k. Strong order 1.0. Avoids use of derivatives (i.e. Runge-Kutta
//              style approach).
//              Ref: Kloeden, P. E. et al (1999): "Numerical Solution of Stochastic
//              Differential Equations", Springer-Verlag.
// Written:     Dave Burke        03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef MILSTEINSOLVER_H
#define MILSTEINSOLVER_H

#include "Solver.h"

class CMilsteinSolver : public CSolver
{
    public:

    CMilsteinSolver(void);
    CMilsteinSolver(const CSolverSettings& rSettings);
    virtual ~CMilsteinSolver(void);

    virtual std::string GetDescription(void);

    protected:

    virtual bool Initialise(void);
    virtual bool Advance(float nTime, float ndt, float* pdW, float* pdZ, float* pXin, float* pXout);
    virtual bool Finish(void);

    bool CheckDiagonalNoise(void);

    float* m_pSupportX;         // support vector
    float* m_pSupportA;         // drift evaluated at support vector
    float** m_ppSupportB;       // diffusion evaluated at support vector
};

#endif // MILSTEINSOLVER_H
