///////////////////////////////////////////////////////////////////////////////
//    Charles River Analytics Inc.
//    Copyright 2002-2003. All Rights Reserved.
//
//    ORIGINAL SYSTEM: The Vision Kit Library
//    SUBSYSTEM: Image
//    $Workfile: PortableGraymap.h $
//    $Revision: 2 $
//    Author(s): V.Ablavsky, J.Pollak, M.Stevens
//
//    Description
//    ===========
//
// Read/Write .pgm files into naive Image container
//
///////////////////////////////////////////////////////////////////////////////

#ifndef PortableGraymap_h
#define PortableGraymap_h

#include "Image.h"
#include "ImageDefs.h"
#include <fstream>
#include <string>

namespace cuj
{

  class PortableGraymapException : public std::exception
  {
    public:
    PortableGraymapException(const std::string &s) : error(s)
    {
    }
    virtual ~PortableGraymapException() throw()
    {
    }
    virtual const char* what() const throw()
    {
      return error.c_str();
    }
    private:
    std::string error;
  };

  class PortableGraymap
  {

    public:

    // read, allocate and return a pgm image
    static inline Image<byte> *read(const std::string &file)
    {

      // open file
      std::ifstream inFile(file.c_str());
      if (! inFile) 
      {
        throw PortableGraymapException("Failed open for read");
      }

      // magic number
      if ((inFile.get() != 'P') || (inFile.get() != '5'))
      {
        throw PortableGraymapException("Image not a P5 pgm");
      }
      inFile.get(); // nl
      
      // dimensions
      size_t w = 0;
      size_t h = 0;
      inFile >> w >> h;
      inFile.get(); // nl
      
      // skip the bit max and assume 255
      size_t maxBit = 0;
      inFile >> maxBit;
      if (maxBit != 255)
      {
        throw PortableGraymapException("Image max bit value not 255");
      }
      inFile.get(); // nl

      // data
      Image<byte> *image = new Image<byte>(w, h);
      inFile.read((char*)(image->begin()), w * h * sizeof(byte));
      
      // done
      inFile.close();

      // return
      return image;
      
    }

    // read, allocate and return a pgm image
    static inline void 
      write(const std::string &file, const Image<byte> &out)
    {
      std::ofstream outFile(file.c_str());
      if (! outFile) 
      {
        throw PortableGraymapException("Faile image open for write");
      }
      outFile << "P5\n" 
              << out.width() << " " 
              << out.height() << "\n255\n";
      outFile.write((char *)(out.begin()), 
                    out.width() * out.height() * sizeof(byte));
      outFile.close();
    }
  };
}

#endif
