///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    RungeKuttaSolver.h
// Package:     SDESolver
// Class:       CRungeKuttaSolver
// Purpose:     Runge-Kutta SDE solver. For simplicity we assume diagonal noise i.e.
//              bjk==0 for j!=k. Strong order 1.5. For autonomous systems.
//              Ref: Kloeden, P. E. et al (1999): "Numerical Solution of Stochastic
//              Differential Equations", Springer-Verlag.
// Written:     Dave Burke        03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef RUNGEKUTTASOLVER_H
#define RUNGEKUTTASOLVER_H

#include "Solver.h"

// For each support vector X, we'll
// store the corresponding drift vector and diffusion matrix (evaluated at X)
typedef struct Support
{
    float* m_pX;
    float* m_pA;
    float** m_ppB;
} SUPPORT;

class CRungeKuttaSolver : public CSolver
{
    public:

    CRungeKuttaSolver(void);
    CRungeKuttaSolver(const CSolverSettings& rSettings);
    virtual ~CRungeKuttaSolver(void);

    virtual std::string GetDescription(void);

    protected:

    virtual bool Initialise(void);
    virtual bool Advance(float nTime, float ndt, float* pdW, float* pdZ, float* pXin, float* pXout);
    virtual bool Finish(void);

    bool CheckDiagonalNoise(void);

    // Require 4 support vector arrays
    SUPPORT* m_pS1;
    SUPPORT* m_pS2;
    SUPPORT* m_pS3;
    SUPPORT* m_pS4;
};

#endif // RUNGEKUTTASOLVER_H
