///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    SolverFactory.h
// Package:     SDESolver
// Class:       CSolverFactory
// Purpose:     Parameterised factory for creating CSolver implementations
// Written:     Dave Burke        03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef SOLVERFACTORY_H
#define SOLVERFACTORY_H

#include "SolverSettings.h"
#include "Solver.h"

class CSolverFactory
{
    public:

    enum SolverType // supported implementations
    {
        EM = 0,
        MILSTEIN = 1,
        RUNGE_KUTTA = 2

        // Add implementations here
    };

    CSolverFactory(void);
    virtual ~CSolverFactory(void);

    static CSolver* CreateSolver(SolverType eType, const CSolverSettings& rSettings);
    static CSolver* CreateSolver(const std::string& rstrType, const CSolverSettings& rSettings);
    static std::string GetName(CSolverFactory::SolverType eType);
    static CSolverFactory::SolverType GetType(const std::string& rstrType);
};

#endif // SOLVERFACTORY_H
