///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    SolverSettings.h
// Package:     SDESolver
// Class:       CSolverSettings
// Purpose:     Base settings for a CSolver subclass
// Written:     Dave Burke        03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include "Util.h"
#include "SolverSettings.h"

#define DEFAULT_OUT_FILE "output.dat"    // default file to write output of solution

///////////////////////////////////////////////////////////////////////////////////////////////
// Function:    CSolverSettings
// Class:       CSolverSettings
// Purpose:     Default constructor
// Inputs:      None
// Outputs:     None
///////////////////////////////////////////////////////////////////////////////////////////////
CSolverSettings::CSolverSettings(void)
: m_nStepSize(0.01), m_nMultiplier(1), m_nT(1),
  m_strOutputFileName(DEFAULT_OUT_FILE), m_strWienerOutputFileName(""), m_nSeed(0)
{
    m_ICs.m_pData = NULL;
    m_ICs.m_nDim = 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////
// Function:    CSolverSettings
// Class:       CSolverSettings
// Purpose:     Copy constructor
// Inputs:      rSettings            object to copy
// Outputs:     None
///////////////////////////////////////////////////////////////////////////////////////////////
CSolverSettings::CSolverSettings(const CSolverSettings& rSettings)
: m_nStepSize(0.01), m_nMultiplier(1), m_nT(1),
  m_strOutputFileName(DEFAULT_OUT_FILE), m_strWienerOutputFileName(""), m_nSeed(0)
{
    m_ICs.m_pData = NULL;
    m_ICs.m_nDim = 0;

    *this = rSettings;
}

///////////////////////////////////////////////////////////////////////////////////////////////
// Function:    ~CSolverSettings
// Class:       CSolverSettings
// Purpose:     Destructor
// Inputs:      None
// Outputs:     None
///////////////////////////////////////////////////////////////////////////////////////////////
CSolverSettings::~CSolverSettings(void)
{
    // Free memory
    if (NULL != m_ICs.m_pData)
    {
        free(m_ICs.m_pData);
        m_ICs.m_pData = NULL;
    }
}

///////////////////////////////////////////////////////////////////////////////////////////////
// Function:    operator=
// Class:       CSolverSettings
// Purpose:     Assignment operator
// Inputs:      rSettings            object to copy from
// Outputs:     returns              this object (to faciliate a=b=c etc)
///////////////////////////////////////////////////////////////////////////////////////////////
CSolverSettings& CSolverSettings::operator=(const CSolverSettings& rSettings)
{
    m_nStepSize = rSettings.m_nStepSize;
    m_nMultiplier = rSettings.m_nMultiplier;
    m_nT = rSettings.m_nT;
    m_strOutputFileName = rSettings.m_strOutputFileName;
    m_strWienerOutputFileName = rSettings.m_strWienerOutputFileName;
    m_nSeed = rSettings.m_nSeed;

    // Deep copy
    if (NULL != m_ICs.m_pData) free(m_ICs.m_pData);
    m_ICs.m_pData = (float*)malloc(sizeof(float) * rSettings.m_ICs.m_nDim);
    for (unsigned int i = 0; i < rSettings.m_ICs.m_nDim; ++i)
    {
        m_ICs.m_pData[i] = rSettings.m_ICs.m_pData[i];
    }

    return *this;
}
