///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    SolverSettings.h
// Package:     SDESolver
// Class:       CSolverSettings
// Purpose:     Base settings for a CSolver subclass
// Written:     Dave Burke        03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef SOLVERSETTINGS_H
#define SOLVERSETTINGS_H

#include <string>

// Dynamical system initial conditions
typedef struct InitialConditions
{
    float* m_pData;
    unsigned int m_nDim;
} ICs;

class CSolverSettings
{
    public:

    CSolverSettings(void);
    CSolverSettings(const CSolverSettings& settings);
    CSolverSettings& operator=(const CSolverSettings& settings);
    virtual ~CSolverSettings(void);

    float m_nStepSize;                                  // step size for Brownian path
    unsigned int m_nMultiplier;                         // solver uses m_nStepSize*m_nMultiplier
    float m_nT;                                         // solve from 0 to m_nT (exclusive of right endpoint)
    ICs m_ICs;                                          // initial conditions
    std::string m_strOutputFileName;                    // output file name for solution
    std::string m_strWienerOutputFileName;              // optional filename to save the Wiener realisation to disk
    unsigned long m_nSeed;                              // optional seed value for number generator
};

#endif // SOLVERSETTINGS_H



