///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    SteinerDynSystem.h
// Package:     SDESolver
// Class:       CDynSystem
// Purpose:     SDE system in Ito form:
//                  dX1 = [dH/dX2 - l*(H)*dH/dX1]dt + [-l*sigma*dH/dX1]dW
//                  dX2 = [-dH/dX1 - l*(H)*dH/dX2]dt + [-l*sigma*dH/dX2]dW              ...1
//
//              where W is Wiener process and formally dW/dt = n(t) white noise, and
//              H is the equation for the Steiner Hypocycloid:
//                  (X1^2+X2^2)^2 + 8*r*X1(3*X2^2-X1^2) + 18*r^2*(X1^2+X2^2)-27*r^4     ...2
//              and l > 0, sigma is noise intensity, r = 1
//
// Written:     Dave Burke        03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef STEINERDYNSYSTEM_H
#define STEINERDYNSYSTEM_H

#include "DynSystem.h"

class CSteinerDynSystem : public CDynSystem
{
    public:

    CSteinerDynSystem(void);
    virtual void GetDriftAndDiffusion(float nTime, float* pXin,
                                      float* pAout, float** ppBout);

    float m_nL;            // l from equation (1) above
    float m_nS;            // sigma from equation (1) above
    float m_nR;            // r from equation (2) above
};

// Factory methods to be used external to shared object
extern "C" CDynSystem* Create(void);
extern "C" void Destroy(CDynSystem* pDynSystem);

#endif // STEINERDYNSYSTEM_H
