///////////////////////////////////////////////////////////////////////////////
//    Charles River Analytics Inc.
//    Copyright 2002-2006. All Rights Reserved.
//
//    ORIGINAL SYSTEM: The Vision Kit Library
//    SUBSYSTEM:      Image
//    $Workfile: Window.h $
//    $Revision: 1 $
//    Author(s): V.Ablavsky, T.Goodsell, J.Pollak, M.Stevens, D. Stouch
//
//    Description
//    ===========
//
// Allows iteration overall sub windows in an image
//
///////////////////////////////////////////////////////////////////////////////

#ifndef Window_h
#define Window_h

#include "SubWindow.h"
#include <iterator>

namespace cuj
{

  //----------------------------------------------------------------------
  template<size_t _SubWidth, size_t _SubHeight, typename _Tp>class Window
  {
    public:

    // simplifying typedef
    typedef Window<_SubWidth, _SubHeight, _Tp> self;
    typedef SubWindow<_SubWidth, _SubHeight, _Tp> SubWindow;

    // the iterator
    class iterator : public std::iterator<std::forward_iterator_tag, _Tp>
    {
      friend class Window<_SubWidth, _SubHeight, _Tp>;
      public:
      
      // dereference to get a subwindow buffer.
      inline SubWindow &operator*()
      {
        subWindow_.updateCenter(x_, y_);
        return subWindow_;
      }
      
      // advance using pre-increment
      inline iterator & operator ++(void)
      {
        ++x_;
        if (x_ == width_)
        {
          x_ = 0;
          ++y_;
        }
        return *this;
      }
      
      // required to test if iteration complete
      inline bool operator!=(const iterator & rhs) const
      {
        return ((x_ != rhs.x_) || (y_ != rhs.y_));
      }
      
      protected:            
      
      private:
      
      // Constructor
      iterator(_Tp *in, size_t width, size_t height) :
        width_(width),
        height_(height),
        subWindow_(in, width, height)
      {
      }
      
      // the current position in the mask
      size_t x_;
      size_t y_;

      // local copy of the size
      size_t width_;
      size_t height_;
      
      // the subwindow
      SubWindow subWindow_;
      
    };

    // constructor 
    Window(_Tp *in, size_t width, size_t height) :
      begin_(in, width, height),
      end_  (in, width, height)
    {
    }
    
    // access 
    inline iterator &begin()
    {
      begin_.x_ = 0;
      begin_.y_ = 0;
      return begin_;
    }
    inline iterator &end()
    {
      end_.x_   = 0;
      end_.y_   = end_.height_;
      return end_;
    }    
    
    private:
    
    // current location
    iterator begin_;
    iterator end_;    
    
  };
  //----------------------------------------------------------------------
  
}
#endif // Window_h
