
//    Date : Sun May 13 13:55:57 EDT 2001
//    Author : Sony E Antony


//    File : ELF_Exception.h
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com


#ifndef __ELF_EXCEPTION_H__
#define __ELF_EXCEPTION_H__

#include<string>
#include<sstream>
#include<exception>
#include<typeinfo>


namespace ELF {

using namespace std ;

class Exception : public std::exception {

  protected :
    string method_ ;
    unsigned int line_ ;
    string file_ ;
    string reason_ ;
    string stacktrace_ ;
    bool initialized_ ;

    Exception* nestedExceptn_ ;

  public:
    Exception() : initialized_( false ), nestedExceptn_( NULL ) {}
    Exception( const Exception &e ) :
        method_( e.method_ ), line_( e.line_ ), file_( e.file_ ),
        reason_( e.reason_ ), stacktrace_( e.stacktrace_ ),
        initialized_( e.initialized_ ),
        nestedExceptn_( ( initialized_ && e.nestedExceptn_ ) ? e.nestedExceptn_->clone() : NULL ) {}

    Exception( unsigned int line, const string &file,
        const string &method, const string &reason, Exception* e = NULL ) :
        line_( line ), file_( file ), method_( method ),
        reason_( reason ), nestedExceptn_( e ) { initialized_ = true ; }
    virtual ~Exception() throw() { if( initialized_ && nestedExceptn_ ) { delete nestedExceptn_ ; } }
    virtual Exception* clone() const = 0 ;

    string file() const { return file_ ; }
    unsigned int line() const { return line_ ; }
    string method() const { return method_ ; }
    void method( const string &s ) { method_ = s ; }
    string reason() const { return reason_ ; }
    void reason( const string &s ) { reason_ = s ; }
    const char* what() const throw() { return typeid( *this ).name() ; }

    void addStackFrame( string s ) {
        if( stacktrace_ == "" ) { stacktrace_ = " " ; return ; }
        if( stacktrace_ == " " ) { stacktrace_ = "" ; }
        else { stacktrace_ += "\n" ; }
        stacktrace_ += ( "    called by " + s ) ;
    }
    string getStackTrace( ) const { return stacktrace_ ; }

    virtual string toString( const string &tab = "" ) const ;

  private:
    void operator = ( const Exception & ) {}
    
};

ostream& operator << ( ostream& os, const Exception &e ) ;


#define DEFINE_EXCEPTION( excptn ) \
        class excptn : public Exception { \
      public: \
        excptn(){}\
        excptn(unsigned int line, const string &file, \
            const string &method, const string &reason, Exception* e = NULL )\
            : Exception( line, file, method, reason, e ) {} \
        Exception* clone() const { Exception* e =\
            new excptn( *this ) ; return e ; }\
    }


#define THROW_EXCEPTION( excptn, msg ) { throw excptn(\
    __LINE__, __FILE__, ELF_FUNC_NAME, msg ) ; }
#define THROW_NESTD_EXCPTN( excptn, msg, nestedE ) {\
    throw excptn( __LINE__, __FILE__, ELF_FUNC_NAME, msg, nestedE.clone() ) ; }
#define THROW_EXCEPTN_WWHERE( excptn, where,  msg ) {\
    throw excptn(  __LINE__, __FILE__, where, msg ) ; }
#define THROW_NESTD_EXCEPTN_WWHERE( excptn, where,  msg, nestedE ) {\
    throw excptn(  __LINE__, __FILE__, where, msg, nestedE.clone() ) ; }

extern string ELF_FUNC_NAME ;        

}
#endif
