
//    Date : Sun May 13 13:55:57 EDT 2001
//    Author : Sony E Antony


//    File : ELF_FileWriterImpl.h
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com


#include<errno.h>
#include<unistd.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<stdio.h>

#include "ELF_MiscUtils.h"
#include "ELF_LogWriter.h"
#include "ELF_Properties.h"


using namespace std ;

namespace ELF {

DEFINE_EXCEPTION( FileWriterImplException ) ;

class FileWriterImpl : public LogWriter {


    string critFilName_, errFilName_, warnFilName_, audtFilName_, debgFilName_, functCallFilName_ ;

  public:
    FileWriterImpl(  const Properties &props ) throw ( Exception ) ;

    ~FileWriterImpl() {}

  private :

    void logCritical_( const string &msg ) throw ( Exception )
        { FileGuard file( openFileForAppend_( critFilName_ ) ) ;
        logMessage_( file.fd(), msg ) ; }
    void logError_( const string &msg ) throw ( Exception )
        { FileGuard file( openFileForAppend_( errFilName_ ) ) ;
        logMessage_( file.fd(), msg ) ; }
    void logWarning_( const string &msg ) throw ( Exception )
        { FileGuard file( openFileForAppend_( warnFilName_ ) ) ;
        logMessage_( file.fd(), msg ) ; }
    void logAudit_( const string &msg ) throw ( Exception )
        { FileGuard file( openFileForAppend_( audtFilName_ ) ) ;
        logMessage_( file.fd(), msg ) ; }
    void logDebug_( const string &msg ) throw ( Exception )
        { FileGuard file( openFileForAppend_( debgFilName_ ) ) ;
        logMessage_( file.fd(), msg ) ; }
    void logFunctCall_( const string &msg ) throw ( Exception )
        { FileGuard file( openFileForAppend_( functCallFilName_ ) ) ;
        logMessage_( file.fd(), msg ) ; }

    inline int openFileForAppend_( const string &file ) throw ( Exception ) {
        int fd ;
        if( -1 == ( fd = open( file.c_str(), O_WRONLY|O_APPEND|O_CREAT, 0666 ) ) ) {
            ostringstream os ;
            os<<"Failed to open File : "<<file<<endl ;
            os<<strerror( errno ) ;
            THROW_EXCEPTN_WWHERE( FileWriterImplException, \
                "int FileWriterImpl::openFileForAppend_( const string &file )"\
                "throw ( Exception )",os.str() ) ;
        }
        return fd ;
    }

    inline void logMessage_( int fd, const string &msg ) throw ( Exception ) {

        if ( -1 == write( fd, msg.c_str(), msg.length() ) ) {
            ostringstream os ;
            os<<"Failed to log Message : "<<msg<<endl ;
            os<<strerror( errno ) ;
            THROW_EXCEPTN_WWHERE( FileWriterImplException, \
                "int FileWriterImpl::logMessage_( int fd, const string &msg )"\
                "throw ( Exception )",os.str() ) ;
        }
    }

    void createEmptyFiles_() throw ( Exception ) ;

};

}
