
//    Date : Sun Sep  1 20:42:50 EDT 2002
//    Author : Sony E Antony


//    File : ELF_LogManager.C
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com

#ifndef SINGLE_THREAD_PROCESS
#include<pthread.h>
#endif

#include<unistd.h>
#include<time.h>
#include<sstream>
#include<iostream>


#include "ELF_LogManager.h"
#include "ELF_MiscUtils.h"
#include "ELF_Properties.h"

using namespace ELF ;

namespace ELF {
    LogManager logManager ;
    }

LogManager::LogManager( ) {

    initialized_ = false ;
    ostringstream os ;
    os<<getpid() ;
    pid_ = os.str() ;

    }

LogManager::~LogManager( ) {

     delete logWriter_ ;
    }

void LogManager::addLogWriter( LoggingType type, Properties props ) throw( Exception ) {

    if( !initialized_ ) {
        THROW_EXCEPTN_WWHERE( LogManagerException,
        "void LogManager::addLogWriter( LoggingType type, Properties props )",
        "Attempt to use Uninitialized LogManager" ) ;
    }
    const char* logTypes[] = { "filebased", "udpbased", "tcpbased" } ;
    logWriter_->addChainedWriter( createLogWriter( logTypes[type], props_ ) ) ;
}

void LogManager::initialize( LoggingType type, PropertyList props ) throw( Exception ) {

    props_ = props ;
    logFile_ = props_.getProperty( "ELF_LOG_MANAGER_LOGFILE_NAME" ) ;
    const char* logTypes[] = { "filebased", "udpbased", "tcpbased" } ;
    logWriter_ = createLogWriter( logTypes[type], props_ ) ;
    string logLevelStr = props_.getProperty( "ELF_DATA_LOGLEVEL" ) ;
    if( logLevelStr == "CRITICAL" ) {
        setDataLogLevel( LOG_CRITICAL ) ;
    }
    else if( logLevelStr == "ERROR" ) {
        setDataLogLevel( LOG_ERROR ) ;
    }
    else if( logLevelStr == "WARNING" ) {
        setDataLogLevel( LOG_WARNING ) ;
    }
    else if( logLevelStr == "AUDIT" ) {
        setDataLogLevel( LOG_AUDIT ) ;
    }
    else if( logLevelStr == "DEBUG" ) {
        setDataLogLevel( LOG_DEBUG ) ;
    }
    else if( logLevelStr == "NONE" ) {
        setDataLogLevel( LOG_NONE ) ;
    }
    else {
        string s = "ERROR : Unknown loglevel specified in Config File \n" ;
        if( props_.getPropertyFileName() != "" ) {
            s += ( "File : " + props_.getPropertyFileName() + "\n" ) ;
        }
        s += ( "Tag : ELF_DATA_LOGLEVEL\n" ) ;
        s += ( "Value : " + logLevelStr + "\n" ) ;
        s += "Permitted Values : CRITICAL|ERROR|WARNING|AUDIT|DEBUG|NONE" ;
        THROW_EXCEPTN_WWHERE( LogManagerException,
        "void LogManager::LogManager::initialize( LoggingType type, Properties props )", s ) ;
    }

    logLevelStr = props_.getProperty( "ELF_FUNC_TRACING_ENABLED" ) ;

    if( logLevelStr == "" ) {
        turnOffFuncCallTracing() ;
    }
    else if( logLevelStr == "yes" ) {
        turnOnFuncCallTracing() ;
    }
    else if( logLevelStr == "no" ) {
        turnOffFuncCallTracing() ;
    }
    else {
        string s = "ERROR : Unknown value specified in properties \n" ;
        if( props_.getPropertyFileName() != "" ) {
            s += ( "File : " + props_.getPropertyFileName() + "\n" ) ;
        }
        s += ( "Tag : ELF_FUNC_LOGGING_ENABLED\n" ) ;
        s += ( "Value : " + logLevelStr + "\n" ) ;
        s += "Permitted Values : yes|no" ;
        THROW_EXCEPTN_WWHERE( LogManagerException,
        "void LogManager::LogManager::initialize( LoggingType type, Properties props )", s ) ;
    }
    ofstream ofs( logFile_.c_str(), ios::out|ios::app ) ;
    if( !ofs.is_open() ) {
        THROW_EXCEPTN_WWHERE( LogManagerException,\
        "void LogManager::LogManager::initialize( LoggingType type, Properties props )", \
            "Failed to write open LogFile : " + logFile_ ) ;
    }
    initialized_ = true ;
    }

void LogManager::setDataLogLevel( DataLogLevel l ) {

    if( initialized_ && getDataLogLevel() == l ) {
        return ;
    }
    if( l == LOG_NONE ) {
        logRoutinePtrs_[0] = logRoutinePtrs_[1] = logRoutinePtrs_[2] =
            logRoutinePtrs_[3] = logRoutinePtrs_[4] = &LogWriter::dummy ;
    }
    else if( l == LOG_CRITICAL ) {
        logRoutinePtrs_[0] = &LogWriter::logCritical ;
        logRoutinePtrs_[1] = logRoutinePtrs_[2] = logRoutinePtrs_[3] =
            logRoutinePtrs_[4] = &LogWriter::dummy ;
    }
    else if( l == LOG_ERROR ) {
        logRoutinePtrs_[0] = &LogWriter::logCritical ;
        logRoutinePtrs_[1] = &LogWriter::logError ;
         logRoutinePtrs_[2] = logRoutinePtrs_[3] = logRoutinePtrs_[4] =
             &LogWriter::dummy ;
    }
    else if( l == LOG_WARNING ) {
        logRoutinePtrs_[0] = &LogWriter::logCritical ;
        logRoutinePtrs_[1] = &LogWriter::logError ;
        logRoutinePtrs_[2] = &LogWriter::logWarning ;
        logRoutinePtrs_[3] = logRoutinePtrs_[4] = &LogWriter::dummy ;
    }
    else if( l == LOG_AUDIT ) {
        logRoutinePtrs_[0] = &LogWriter::logCritical ;
        logRoutinePtrs_[1] = &LogWriter::logError ;
        logRoutinePtrs_[2] = &LogWriter::logWarning ;
        logRoutinePtrs_[3] = &LogWriter::logAudit ;
        logRoutinePtrs_[4] = &LogWriter::dummy ;
    }
    else if( l == LOG_DEBUG ) {
        logRoutinePtrs_[0] = &LogWriter::logCritical ;
        logRoutinePtrs_[1] = &LogWriter::logError ;
        logRoutinePtrs_[2] = &LogWriter::logWarning ;
        logRoutinePtrs_[3] = &LogWriter::logAudit ;
        logRoutinePtrs_[4] = &LogWriter::logDebug ;
    }
}

void LogManager::logMessage_( const string& msg, int level ) throw( ) {

    try {
        if( ! initialized_ ) {
            THROW_EXCEPTN_WWHERE( LogManagerException,
                "void LogManager::logMessage_( const string& msg, int level )",
                "Attempt to use uninitialized LogManager" ) ;
        }
        ostringstream os ;

#ifndef SINGLE_THREAD_PROCESS
        os<<" |THREAD ID = "<<pthread_self() ;
#endif

        string packet = msg + os.str() + " |PID = "+ pid_ +" | TIME= " + currentDateTime() + "\n" ;
        (logWriter_->*logRoutinePtrs_[level])( packet ) ;
    }
    catch( Exception &e ) {
        writeLogManagerMsgs_( e.toString() ) ;
    }

}


LogManager::FunctCallMonitor::FunctCallMonitor( bool &trcngIsOn, const string &sig ): trcngIsOn_( trcngIsOn ), funSig_( sig ) {

    if( !trcngIsOn_ ) {
        return ;
    }
    logManager.logFunc( "METHOD:" + funSig_ + ":ENTRY" ) ;
}

LogManager::FunctCallMonitor::~FunctCallMonitor( ) {

    if( !trcngIsOn_ ) {
        return ;
    }
    logManager.logFunc( "METHOD:" + funSig_ + ":EXIT" ) ;
}


void LogManager::writeLogManagerMsgs_( const string  &msg ) throw() {

    ostringstream os ;
    os<<"======================= LOG MANAGER MSG BEGIN ======================"<<endl ;
    os<<"TIME : "<<currentDateTime()<<" PID : "<<pid_<<endl ;
    os<<msg<<endl ;
    os<<"======================= LOG MANAGER MSG END ======================"<<endl ;
    ofstream ofs( logFile_.c_str(), ios::out|ios::app ) ;
    if( ofs.is_open() ) {
        ofs<<os.str() ;
    }
    else {
        cerr<<os.str() ;
    }
}
