
//    Date : Sun May 13 13:55:57 EDT 2001
//    Author : Sony E Antony


//    File : ELF_LogWriter.C
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com



#include "ELF_LogWriter.h"
#include "ELF_FileWriterImpl.h"
#include "ELF_TcpLogWriterImpl.h"
#include "ELF_UdpLogWriterImpl.h"


using namespace std ;
using namespace ELF ;

LogWriter* ELF::createLogWriter( const string &type, const Properties &props ) throw ( Exception ) {

    if( type == "filebased" ) {
        return new FileWriterImpl( props ) ;
    }
    else if( type == "tcpbased" ) {
        return new TcpLogWriterImpl( props ) ;
    }
    else if( type == "udpbased" ) {
        return new UdpLogWriterImpl( props ) ;
    }

    else {
        THROW_EXCEPTN_WWHERE( LogWriterException, "LogWriter* createLogWriter( " \
            "const string &type,  const Properties &props )",\
            "Unknown type of logwriter type specified : " + type + \
            "\nKnown types : filebased, udpbased, tcpbased" ) ;
    }

}
void LogWriter::addChainedWriter( LogWriter* w ) {

    if( chainedWriter_ ) {
        chainedWriter_->addChainedWriter( w ) ;
        return ;
    }
    chainedWriter_ = w ;
}

bool LogWriter::removeTailEndChainedWriter() {

    if( !chainedWriter_ ) {
        return false ;
    }
    if( !chainedWriter_->removeTailEndChainedWriter() ) {
        delete chainedWriter_ ;
        chainedWriter_ = NULL ;
    }
    return true ;

}
