
//    Date : Sun May 13 13:55:57 EDT 2001
//    Author : Sony E Antony


//    File : ELF_LogWriter.h
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com


#ifndef __ELF_LOG_WRITER_H__
#define __ELF_LOG_WRITER_H__

#include<string>
#include<map>

#include"ELF_Properties.h"
#include"ELF_Exception.h"


namespace ELF {


using namespace std ;

DEFINE_EXCEPTION( LogWriterException ) ;

class LogWriter {

    LogWriter* chainedWriter_ ;

  public:

    LogWriter():chainedWriter_( NULL ) {}
    virtual ~LogWriter() {}

    void addChainedWriter( LogWriter* w ) ;
    bool removeTailEndChainedWriter( ) ;

    inline void logCritical( const string &msg ) throw ( Exception ) {
        if( chainedWriter_ ) { chainedWriter_->logCritical( msg ) ; }
        logCritical_( msg ) ;
    }
    inline void logError( const string &msg ) throw ( Exception ) {
        if( chainedWriter_ ) { chainedWriter_->logError( msg ) ; }
        logError_( msg ) ;
    }
    inline void logWarning( const string &msg ) throw ( Exception ) {
        if( chainedWriter_ ) { chainedWriter_->logWarning( msg ) ; }
        logWarning_( msg ) ;
    }
    inline void logAudit( const string &msg ) throw ( Exception ) {
        if( chainedWriter_ ) { chainedWriter_->logAudit( msg ) ; }
        logAudit_( msg ) ;
    }
    inline void logDebug( const string &msg ) throw ( Exception ) {
        if( chainedWriter_ ) { chainedWriter_->logDebug( msg ) ; }
        logDebug_( msg ) ;
    }
    inline void logFunctCall( const string &msg ) throw ( Exception ) {
        if( chainedWriter_ ) { chainedWriter_->logFunctCall( msg ) ; }
        logFunctCall_( msg ) ;
    }
    void dummy( const string & ) {}

  private :
    virtual void logCritical_( const string &msg ) throw ( Exception ) = 0 ;
    virtual void logError_( const string &msg ) throw ( Exception ) = 0 ;
    virtual void logWarning_( const string &msg ) throw ( Exception ) = 0 ;
    virtual void logAudit_( const string &msg ) throw ( Exception ) = 0 ;
    virtual void logDebug_( const string &msg ) throw ( Exception ) = 0 ;
    virtual void logFunctCall_( const string &msg ) throw ( Exception ) = 0 ;

};

LogWriter* createLogWriter( const string &type, const Properties &props = appConfig ) throw ( Exception ) ;

}
#endif
