
//    Date : Sun Sep  1 20:09:28 EDT 2002
//    Author : Sony E Antony


//    File : ELF_Properties.C
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com


#include<stdlib.h>

#include"ELF_Properties.h"

using namespace ELF ;

namespace ELF {

    Properties appConfig ;
}

Properties::Properties(
    const PropertyList &proplist ) throw ( Exception ) {

    initialize( proplist ) ;
}

Properties::Properties(
    const string &cfgFileName ) throw ( Exception ) {

    initialize( cfgFileName ) ;
}

void Properties::reInitialize() throw ( Exception ) {

    if( cfgFileName_ == "" ) {
        THROW_EXCEPTN_WWHERE( PropertiesException, \
            "void Properties::reInitialize()", \
            "Attempt to reinitialize with empty configuration file name" ) ;
    }
    initialize( cfgFileName_ ) ;
}

void Properties::initialize( const string &cfgFileName ) throw ( Exception ) {

    if( cfgFileName == "" ) {
        THROW_EXCEPTN_WWHERE( PropertiesException, \
            "void Properties::initialize( const string &cfgFileName )", \
            "Attempt to reinitialize with empty configuration file name" ) ;
    }
    initialized_ = false ;
    cfgFileName_ = cfgFileName ;
    readPropertiesFile_( ) ;
    setProperty( "ELF_PROPERTIES_FILE_NAME", cfgFileName ) ;
    initialized_ = true ;
    applName_ = getOptionalProperty( "ELF_APPL_NAME" ) ;
}

void Properties::initialize( const PropertyList proplist ) throw ( Exception ) {

    initialized_ = false ;
    properties_ = proplist ;
    initialized_ = true ;
    cfgFileName_ = getOptionalProperty( "ELF_PROPERTIES_FILE_NAME" ) ;
    applName_ = getOptionalProperty( "ELF_APPL_NAME" ) ;
}

void Properties::readPropertiesFile_() throw ( Exception ) {


    ifstream cfgFileStr( cfgFileName_.c_str() ) ;
    if( !cfgFileStr.is_open() ) {
        THROW_EXCEPTN_WWHERE( PropertiesException, \
            "void Properties::readPropertiesFile_( )", \
            "Error opening Config File : " + cfgFileName_ ) ;
    }

    string line ;

    properties_.clear() ;
    while( getline( cfgFileStr, line )  ) {

        int i = line.find( '=' ) ;
        properties_.insert( PropertyList::value_type( line.substr( 0, i ), line.substr( i+1 ) ) ) ;
    }

    initialized_ = true ;
}

string Properties::getOptionalProperty( const string  &property ) const
        throw( Exception ) {

    if( !initialized_ ) {
        THROW_EXCEPTN_WWHERE( PropertiesException, "string Properties::getOptionalProperty("\
            "const string  &property )", "Uninitialized Cfg File Reader" ) ;
    }
    PropertyList::const_iterator it ;
    if( ( it = properties_.find( property ) ) != properties_.end() ) {
        return it->second ;
    }
    return "" ;
}

string Properties::getProperty( const string  &property ) const
        throw( Exception ) {

    string val = getOptionalProperty( property ) ;
    if( val == "" ) {
        THROW_EXCEPTN_WWHERE( PropertiesException, "string Properties::getCfgValue("\
            "const string  &property )", "Configuration File Error : Parameter " + property +  \
            " Not set in Cfg File :" + cfgFileName_ ) ;
    }
    return val ;
}
