//    Date : Sun May 13 13:55:57 EDT 2001
//    Author : Sony E Antony


//    File : ELF_TcpLogWriterImpl.C
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com

#include<errno.h>
#include<unistd.h>
#include<sys/types.h>
#include<sys/socket.h>
#include<sys/stat.h>
#include<netinet/in.h>
#include<arpa/inet.h>
#include<fcntl.h>
#include<stdio.h>
#include<stdlib.h>
#include<strings.h>


#include "ELF_MiscUtils.h"
#include "ELF_ConnectionTypes.h"
#include "ELF_TcpLogWriterImpl.h"



using namespace std ;
using namespace ELF ;

TcpLogWriterImpl::TcpLogWriterImpl( const Properties &props ) throw ( Exception ) {

    destIp_ = props.getProperty( "ELF_LOG_SERVER_TCP_IP_ADDR" ) ;
    destPort_ = props.getProperty( "ELF_LOG_SERVER_TCP_PORT" ) ;

    critFd_ = errFd_ = warnFd_ = audtFd_ = debgFd_ = funcalFd_ = -1 ;
    initialize() ;
} 
    
void TcpLogWriterImpl::reInitialize( const string &destIp, const string &destPort )
    throw ( Exception ) {

    destIp_ = destIp ;
    destPort_ = destPort ;
    close( critFd_ ) ;
    close( errFd_ ) ;
    close( warnFd_ ) ;
    close( audtFd_ ) ;
    close( debgFd_ ) ;
    close( funcalFd_ ) ;
    initialize() ;
}   

void TcpLogWriterImpl::initialize() throw ( Exception ) { 

    FileGuard g1( critFd_ = openSocket_( destIp_, destPort_ ) ) ;
    FileGuard g2( errFd_ = openSocket_( destIp_, destPort_ ) ) ;
    FileGuard g3( warnFd_ = openSocket_( destIp_, destPort_ ) ) ;
    FileGuard g4( audtFd_ = openSocket_( destIp_, destPort_ ) ) ;
    FileGuard g5( debgFd_ = openSocket_( destIp_, destPort_ ) ) ;
    FileGuard g6( funcalFd_ = openSocket_( destIp_, destPort_ ) ) ;

    sendMessage_( critFd_, string( 1,  ConnectionTypes::CRITICAL ) ) ;
    sendMessage_( errFd_, string( 1,  ConnectionTypes::ERROR ) ) ;
    sendMessage_( warnFd_, string( 1,  ConnectionTypes::WARNING ) ) ;
    sendMessage_( audtFd_, string( 1,  ConnectionTypes::AUDIT ) ) ;
    sendMessage_( debgFd_, string( 1,  ConnectionTypes::DEBUG ) ) ;
    sendMessage_( funcalFd_, string( 1,  ConnectionTypes::FUNCALL ) ) ;
    g1.noClose() ;
    g2.noClose() ;
    g3.noClose() ;
    g4.noClose() ;
    g5.noClose() ;
    g6.noClose() ;
}


int TcpLogWriterImpl::openSocket_( const string &destIp, const string &destPort ) throw ( Exception ) {

    const char* portstr = destPort.c_str() ;
    char *cp ;
    errno = 0 ;
    int portint = strtol( portstr, &cp, 10 ) ;
    if( ( *cp != '\0' ) || ( portint <= 0 ) || ( portint > 65536 ) ) {
        ostringstream os ;
        os<<"Failed to convert the port to a valid TCP port ( 0 < validport < 65536 ) : "<<destPort<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogWriterImplException, \
            "TcpLogWriterImpl::openSocket_( string destIp, string destPort )",\
            os.str() ) ;
        }
    
    struct sockaddr_in dest ;
    bzero( &dest, sizeof( dest ) ) ;
    dest.sin_family = AF_INET ;
    dest.sin_port =  htons(  (uint16_t) portint ) ;
    if( -1 == inet_pton( AF_INET, destIp.c_str(), &dest.sin_addr ) ) {
        ostringstream os ;
        os<<"Failed to convert the address to an IP address : "<<destIp<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogWriterImplException, \
            "TcpLogWriterImpl( string destIp, string destPort )",\
            os.str() ) ;
        }
    int sd ;
    if( -1 == ( sd = socket( AF_INET, SOCK_STREAM, 0 ) ) ) {
        ostringstream os ;
        os<<"Failed to create TCP stream socket : "<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogWriterImplException, \
            "TcpLogWriterImpl( string destIp, string destPort )",\
            os.str() ) ;
        }
    socklen_t len = sizeof( dest ) ;
    if( -1 == connect( sd, (struct sockaddr*)&dest, len ) ) {
        ostringstream os ;
        os<<"Failed to connect to the remote TCP host : "<<destIp<<":"<<destPort<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogWriterImplException, \
            "TcpLogWriterImpl( string destIp, string destPort )",\
            os.str() ) ;
        }
    return sd ;
}
