//    Date : Sun May 13 13:55:57 EDT 2001
//    Author : Sony E Antony


//    File : ELF_TcpLogWriterImpl.h
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com


#ifndef __ELF_TCP_LOG_WRITER_IMPL_H__
#define __ELF_TCP_LOG_WRITER_IMPL_H__


#include "ELF_LogWriter.h"
#include "ELF_MiscUtils.h"


using namespace std ;

namespace ELF {

DEFINE_EXCEPTION( TcpLogWriterImplException ) ;

class TcpLogWriterImpl : public LogWriter {

    string destIp_, destPort_ ;
    int critFd_, errFd_, warnFd_, audtFd_, debgFd_, funcalFd_ ;
  public:
    TcpLogWriterImpl(  const Properties &props ) throw ( Exception ) ;
    ~TcpLogWriterImpl() {}
    void reInitialize( const string &destIp, const string &destPort )
        throw ( Exception ) ;
    void initialize() throw ( Exception ) ;

  private :

    void logCritical_( const string &msg ) throw ( Exception )
        { sendMessage_( critFd_, msg ) ; }
    void logError_( const string &msg ) throw ( Exception )
        { sendMessage_( errFd_, msg ) ; }
    void logWarning_( const string &msg ) throw ( Exception )
        { sendMessage_( warnFd_, msg ) ; }
    void logAudit_( const string &msg ) throw ( Exception )
        { sendMessage_( audtFd_, msg ) ; }
    void logDebug_( const string &msg ) throw ( Exception )
        { sendMessage_( debgFd_, msg ) ; }
    void logFunctCall_( const string &msg ) throw ( Exception )
        { sendMessage_( funcalFd_, msg ) ; }


    inline void sendMessage_( int fd, const string &msg ) throw ( Exception ) {

        if ( -1 == writen( fd, msg.c_str(), msg.length() ) ) {
            ostringstream os ;
            os<<"Failed to send Message : "<<msg<<endl ;
            os<<strerror( errno ) <<endl ;
            os<<"Socket closed" <<endl ;
            close( fd ) ;
            THROW_EXCEPTN_WWHERE( TcpLogWriterImplException, \
                "void TcpLogWriterImpl::_sendMessage_( int fd, string msg ) throw ( Exception )",\
                os.str() ) ;
        }
    }

    int openSocket_( const string &destIp, const string &destPort ) throw ( Exception ) ;
};

}

#endif
