//    Date : Sun May 13 13:55:57 EDT 2001
//    Author : Sony E Antony


//    File : ELF_UdpLogWriterImpl.C
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com

#include<errno.h>
#include<unistd.h>
#include<sys/types.h>
#include<sys/socket.h>
#include<sys/stat.h>
#include<netinet/in.h>
#include<arpa/inet.h>
#include<fcntl.h>
#include<stdio.h>
#include<stdlib.h>
#include<strings.h>


#include "ELF_MiscUtils.h"
#include "ELF_UdpLogWriterImpl.h"



using namespace std ;
using namespace ELF ;


UdpLogWriterImpl::UdpLogWriterImpl( const Properties &props ) throw ( Exception ) {

    destIp_ = props.getProperty( "ELF_LOG_SERVER_UDP_IP_ADDR" ) ;
    destPort_ = props.getProperty( "ELF_LOG_SERVER_UDP_PORT" ) ;

    sd_ = -1 ;
    initialize() ;
}

void UdpLogWriterImpl::reInitialize( const string &destIp, const string &destPort )
        throw ( Exception ) {

    destIp_ = destIp ;
    destPort_ = destPort ;
    close( sd_ ) ;
    initialize() ;
}

void UdpLogWriterImpl::initialize() throw ( Exception ) {

    const char* portstr = destPort_.c_str() ;
    char *cp ;
    errno = 0 ;
    int portint = strtol( portstr, &cp, 10 ) ;
    if( ( *cp != '\0' ) || ( portint <= 0 ) || ( portint > 65536 ) ) {
        ostringstream os ;
        os<<"Failed to convert the port to a valid UDP port ( 0 < validport < 65536 ) : "<<destPort_<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( UdpLogWriterImplException, \
            "UdpLogWriterImpl::initialize()",\
            os.str() ) ;
    }
    
    struct sockaddr_in dest ;
    bzero( &dest, sizeof( dest ) ) ;
    dest.sin_family = AF_INET ;
    dest.sin_port =  htons(  (uint16_t) portint ) ;
    if( -11 == inet_pton( AF_INET, destIp_.c_str(), &dest.sin_addr ) ) {
        ostringstream os ;
        os<<"Failed to convert the address to an IP address : "<<destIp_<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( UdpLogWriterImplException, \
            "UdpLogWriterImpl( string destIp, string destPort )",\
            os.str() ) ;
    }
    if( -1 == ( sd_ = socket( AF_INET, SOCK_DGRAM, 0 ) ) ) {
        ostringstream os ;
        os<<"Failed to create UDP datagram socket : "<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( UdpLogWriterImplException, \
            "UdpLogWriterImpl( string destIp, string destPort )",\
            os.str() ) ;
    }
    FileGuard guard( sd_ ) ;
    socklen_t len = sizeof( dest ) ;
    if( -1 == connect( sd_, (struct sockaddr*)&dest, len ) ) {
        ostringstream os ;
        os<<"Failed to set the dest UDP address : "<<destIp_<<":"<<destPort_<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( UdpLogWriterImplException, \
            "UdpLogWriterImpl( string destIp, string destPort )",\
            os.str() ) ;
    }
    guard.noClose() ;
}
