//    Date : Sun May 13 13:55:57 EDT 2001
//    Author : Sony E Antony


//    File : ELF_UdpLogWriterImpl.h
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com

#ifndef __ELF_UDP_LOG_WRITER_IMPL_H__
#define __ELF_UDP_LOG_WRITER_IMPL_H__


#include "ELF_ConnectionTypes.h"
#include "ELF_LogWriter.h"


using namespace std ;

namespace ELF {

DEFINE_EXCEPTION( UdpLogWriterImplException ) ;

class UdpLogWriterImpl : public LogWriter {

    int sd_ ;
    string destIp_, destPort_ ;

  public:
    UdpLogWriterImpl( const Properties &props ) throw ( Exception ) ;
    ~UdpLogWriterImpl() {}
    void reInitialize( const string &destIp, const string &destPort )
        throw ( Exception ) ;
    void initialize() throw ( Exception ) ;

  private :

    void logCritical_( const string &msg ) throw ( Exception )
        { sendMessage_( ConnectionTypes::CRITICAL, msg ) ; }
    void logError_( const string &msg ) throw ( Exception )
        { sendMessage_( ConnectionTypes::ERROR, msg ) ; }
    void logWarning_( const string &msg ) throw ( Exception )
        { sendMessage_( ConnectionTypes::WARNING, msg ) ; }
    void logAudit_( const string &msg ) throw ( Exception )
        { sendMessage_( ConnectionTypes::AUDIT, msg ) ; }
    void logDebug_( const string &msg ) throw ( Exception )
        { sendMessage_( ConnectionTypes::DEBUG, msg ) ; }
    void logFunctCall_( const string &msg ) throw ( Exception )
        { sendMessage_( ConnectionTypes::FUNCALL, msg ) ; }


    inline void sendMessage_( char type, string msg ) throw ( Exception ) {

        msg = type + msg ;
        if ( -1 == write( sd_, msg.c_str(), msg.length() ) ) {
            ostringstream os ;
            os<<"Failed to send Message : "<<msg<<endl ;
            os<<strerror( errno ) ;
            os<<"Socket closed" <<endl ;
            close( sd_ ) ;
            THROW_EXCEPTN_WWHERE( UdpLogWriterImplException, \
                "void sendMessage_( char type, string msg ) throw ( Exception )",\
                os.str() ) ;
        }
    }

};

}

#endif
