
//    Date : Sun Sep  1 20:09:28 EDT 2002
//    Author : Sony E Antony


//    File : ELF_LogServer.C
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com

#include<sys/types.h>
#include<unistd.h>

#include "ELF_LogServer.h"
#include "ELF_MiscUtils.h"
#include "ELF_TcpLogServerImpl.h"
#include "ELF_UdpLogServerImpl.h"


using namespace ELF ;

using namespace std ;

namespace ELF {

    LogServer *logServer ; 
}

LogServer::LogServer( ) throw ( Exception ) {

    logFile_ = appConfig.getProperty( "ELF_LOG_SERVER_LOGFILE_NAME" ) ;

    logWriter_ = createLogWriter( "filebased" ) ;
}

void LogServer::shutdown() {

    writeLogServerMsgs( "LogServer shutting down on request" ) ;
    _exit( 0 ) ; //system call _exit(). Ordinary exit() is not async signal safe.
}

void LogServer::writeLogServerMsgs( const string  &msg ) throw() {

    ostringstream os ;
    os<<"======================= LOG SERVER MSG BEGIN ======================"<<endl ;
    os<<"TIME : "<<currentDateTime()<<" PID : "<<getpid()<<endl ;
    os<<msg<<endl ;
    os<<"======================= LOG SERVER MSG END ======================"<<endl ;
    ofstream ofs( logFile_.c_str(), ios::out|ios::app ) ;
    if( ofs.is_open() ) {
        ofs<<os.str() ;
    }
    else {
        cerr<<os.str() ;
    }
}

LogServer* ELF::createLogServer( const string &type ) throw ( Exception ) {

    if( type == "udpbased" ) {
        return new UdpLogServerImpl ;
    }
    if( type == "tcpbased" ) {
        return new TcpLogServerImpl ;
    }
    else {
        ostringstream os ;
        os<<"Unknown Logserver type specified : "<<type <<endl ;
        os<<"Known types are 'udpbased' & 'tcpbased'"<<endl ;
        THROW_EXCEPTN_WWHERE( LogServerException,
            "LogServer* createLogServer( const string &type )", os.str() ) ;
    }

}
