
//    Date : Sun Sep  1 20:09:28 EDT 2002
//    Author : Sony E Antony
//
//
//    File : ELF_LogServerMain.h
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com
//
#include<unistd.h>

#include<sstream>

#include "ELF_Exception.h"
#include "ELF_SignalManager.h"
#include "ELF_Properties.h"
#include "ELF_LogServer.h"

using namespace std ;
using namespace ELF ;

void showUsageAndExit( string binName ) {
    cerr<<"Usage : "<<binName<<" <-t|-u> -f <config_file>"<<endl ;
    exit( 1 ) ;
}

void initialize( int argc, char *argv[] ) {

    bool logserverTypeSpecified = false ;
    bool udpTypeLogserver ;
    bool confilespecified = false ;

    int ch ;
    string cfgFileName ;
    opterr = 0 ;
    while ( ( ch = getopt( argc, argv, "tuf:") ) != -1 ) {
        switch( ch ) {
            case 'u': {
                if( logserverTypeSpecified ) {
                    showUsageAndExit( argv[0] ) ;
                }
                logserverTypeSpecified = true ;
                udpTypeLogserver = true ;
                break ;
            }
            case 't': {
                if( logserverTypeSpecified ) {
                    showUsageAndExit( argv[0] ) ;
                }
                logserverTypeSpecified = true ;
                udpTypeLogserver = false ;
                break ;
            }
            case 'f': {
                if( confilespecified ) {
                    showUsageAndExit( argv[0] ) ;
                }
                confilespecified = true ;
                cfgFileName = optarg ;
                break ;
            }
            case '?': 
            default : {
                showUsageAndExit( argv[0] ) ;
            }
        }
    }
    if( ! logserverTypeSpecified || !confilespecified ) {
        showUsageAndExit( argv[0] ) ;
    }
    appConfig.initialize( cfgFileName ) ;
    logServer = createLogServer( udpTypeLogserver ? "udpbased" : "tcpbased" ) ;
}


int main( int argc, char *argv[] ) {
    try {
        initialize( argc, argv ) ;

        SignalManager sigmanager ;
        logServer->run( &sigmanager ) ;
    }
    catch( Exception &e ) {

            ostringstream os ;
            os<<"Unexpected Exception caught in main() : "<<endl<<e<<endl ;
            os<<"Logserver Terminating..."<<endl ;
            if( logServer ) {
                logServer->writeLogServerMsgs( os.str() ) ;
            }
            else{
                cerr<<os.str() ;
            }
            exit( 2 ) ;
        }
    catch( ... ) {

            ostringstream os ;
            os<<"Unexpected Unknown exception caught in main()"<<endl ;
            os<<"Logserver Terminating..."<<endl ;
            if( logServer ) {
                logServer->writeLogServerMsgs( os.str() ) ;
            }
            else{
                cerr<<os.str() ;
            }
            exit( 2 ) ;
        }
    return 0 ;
}
