
//    Date : Sun May 13 13:55:57 EDT 2001
//    Author : Sony E Antony


//    File : ELF_SignalManager.C
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com

#include<errno.h>

#include "ELF_LogServer.h"
#include "ELF_SignalManager.h"


using namespace ELF ;

extern "C" void shutdownLogServer( int signo ) {
    if( logServer ) {
        logServer->shutdown() ;
    }
    else {
        exit( 0 ) ;
    }
}

SignalManager::SignalManager() throw( Exception ) {

    sigemptyset( &sigmask_ ) ;
    sigaddset( &sigmask_, SIGHUP ) ;
    if( -1 == sigprocmask( SIG_BLOCK, &sigmask_, &oldSigmask_ ) ) {
        ostringstream os ;
        os<<"Failed to set signal mask for SIGHUP for the process : "<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( SignalManagerException,\
            "SignalManager::SignalManager()", os.str() ) ;
        }
    struct sigaction action ;
    action.sa_handler = &shutdownLogServer ;
    sigemptyset( &action.sa_mask ) ;
    action.sa_flags = SA_RESTART ;
    if( -1 == sigaction( SIGHUP, &action, NULL ) ) {
        ostringstream os ;
        os<<"Failed to set signal handler for signal : SIGHUP "<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( SignalManagerException,\
            "SignalManager::SignalManager()", os.str() ) ;
        }
}

SignalManager::~SignalManager() {
    sigprocmask( SIG_SETMASK, &oldSigmask_, NULL ) ;
}

void SignalManager::preBlock( void* data ) {

    sigprocmask( SIG_UNBLOCK, &sigmask_, NULL ) ;
}

void SignalManager::postBlock( void* data ) {

    sigprocmask( SIG_BLOCK, &sigmask_, NULL ) ;
}
