
//    Date : Sun Sep  1 20:09:28 EDT 2002
//    Author : Sony E Antony


//    File : ELF_TcpLogServerImpl.C
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com

#include<limits.h>

#include<sys/types.h>
#include<sys/socket.h>
#include<sys/poll.h>
#include<netinet/in.h>
#include<arpa/inet.h>
#include<unistd.h>
#include<strings.h>
#include<errno.h>

#include "ELF_ConnectionTypes.h"
#include "ELF_TcpLogServerImpl.h"
#include "ELF_MiscUtils.h"


using namespace ELF ;

using namespace std ;


namespace ELF {
    enum TcpLogServerImplFunPtrIndices {
        CRITICAL = 0 ,
        ERROR = 1 ,
        WARNING = 2 ,
        AUDIT = 3 ,
        DEBUG = 4 ,
        FUNCT_CALL = 5 ,
        UNDECIDED = 6
    };
}

TcpLogServerImpl::~TcpLogServerImpl( ) {

    for( int i = 1 ; i < OPEN_MAX ; i++ ) {
        close( sockets_[i].fd  ) ;
    }
}

TcpLogServerImpl::TcpLogServerImpl( ) throw ( Exception ) {

    logFunctPtrs_[CRITICAL] = &TcpLogServerImpl::logCritical_ ;
    logFunctPtrs_[ERROR] = &TcpLogServerImpl::logError_ ;
    logFunctPtrs_[WARNING] = &TcpLogServerImpl::logWarning_ ;
    logFunctPtrs_[AUDIT] = &TcpLogServerImpl::logAudit_ ;
    logFunctPtrs_[DEBUG] = &TcpLogServerImpl::logDebug_ ;
    logFunctPtrs_[FUNCT_CALL] = &TcpLogServerImpl::logFunctCall_ ;
    logFunctPtrs_[UNDECIDED] = &TcpLogServerImpl::decideConnType_ ;
    string port = appConfig.getProperty( "ELF_LOG_SERVER_TCP_PORT" ) ;
    const char* portstr = port.c_str() ;
    char* cp ;
    port_ = strtol( portstr, &cp, 10 ) ;
    if( ( *cp != '\0' ) || ( port_ <= 0 ) || ( port_ > 65536 ) ) { 
        ostringstream os ;
        os<<"Failed to convert the port to a valid TCP port ( 0 < validport < 65536 ) : "<<port<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, \
            "UdpLogServerImpl::UdpLogServerImpl( )", os.str() ) ; 
    }
    setupSocket_() ;
}

void TcpLogServerImpl::setupSocket_() throw ( Exception ) {

    struct sockaddr_in servsock ;
    if( -1 == ( listenSd_ = socket( AF_INET, SOCK_STREAM, 0 ) ) ) {
        ostringstream os ;
        os<<"Failed to create TCPv4 STREAM socket : "<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, "void TcpLogServerImpl::setupSocket_()", os.str() ) ;
    }
    FileGuard guard( listenSd_ ) ;
    bzero( &servsock, sizeof( servsock ) ) ;
    servsock.sin_family = AF_INET ;
    servsock.sin_addr.s_addr = htonl( INADDR_ANY ) ;
    servsock.sin_port = htons( port_ ) ;
    int on = 1 ;
    socklen_t len = sizeof( on ) ;
    if( -1 == setsockopt( listenSd_, SOL_SOCKET, SO_REUSEADDR, &on, len ) ) {
        ostringstream os ;
        os<<"Failed to set REUSEADDR socket option for socket at port : "<< port_<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, "void TcpLogServerImpl::setupSocket_()", os.str() ) ;
    }
    if( -1 == bind( listenSd_, (struct sockaddr*) &servsock, sizeof( servsock ) ) ) {
        ostringstream os ;
        os<<"Failed to bind() socket to port : "<< port_<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, "void TcpLogServerImpl::setupSocket_()", os.str() ) ;
    }

    if( -1 == listen( listenSd_, 5 ) ) {
        ostringstream os ;
        os<<"Failed to set connection backlog for listening socket : Port = "<< port_<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, "void TcpLogServerImpl::setupSocket_()", os.str() ) ;
    }

    guard.noClose() ;

    sockets_[0].fd = listenSd_ ;
    sockets_[0].events = POLLIN ;
    for( int i = 1 ; i < OPEN_MAX ; i++ ) {
        sockets_[i].fd = -1;
    }
    highestIndex_ = 0 ;
}

bool TcpLogServerImpl::wait4ClientAction_( unsigned int &numRdRdySocs ) throw ( Exception ) {

    int numReadyConns ;
    {
        BlockingOperationSentryGuard guard( callbackObj_ ) ;
        if( -1 == ( numReadyConns = poll( sockets_, highestIndex_ + 1, -1 ) ) ) {
            ostringstream os ;
            os<<"Failed poll() on set of sockets"<<endl ;
            os<<strerror( errno ) ;
            THROW_EXCEPTN_WWHERE( TcpLogServerImplException,
                "bool TcpLogServerImpl::wait4ClientAction_()", os.str() ) ;
        }
    }

    if( sockets_[0].revents & POLLIN ) {
        numRdRdySocs = --numReadyConns ;
        return true ;
    }
    else {
        numRdRdySocs = numReadyConns ;
        return false ;
    }
}

void TcpLogServerImpl::acceptNewConnection_() throw ( Exception ) {

    struct sockaddr_in client ;
    socklen_t addrlen = sizeof( client ) ;

    int newConn ;
    if( -1 == ( newConn = accept( listenSd_, (struct sockaddr *) &client, &addrlen ) ) ) {
        ostringstream os ;
        os<<"Failed to accept the client connection request on the listening socket"<<endl ;
        os<<strerror( errno ) ;
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, "void TcpLogServerImpl::acceptNewConnection_()", os.str() ) ;
    }
    addNewConnection_( client, newConn ) ;
}

void TcpLogServerImpl::addNewConnection_( struct sockaddr_in &client, int sd ) throw ( Exception ) {
    
    FileGuard guard( sd ) ;
    if( sdFptrRelations_.end() != sdFptrRelations_.find( sd ) ) {
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, "void TcpLogServerImpl::addNewConnection_( int sd )", \
            "FATAL : Already existing socket descriptor attempted to be added to the processing function pointer table" ) ;
    }
    int i ;
    for( i = 1 ; i < OPEN_MAX ; i++ ) {
        if( sockets_[i].fd < 0 ) {
            sockets_[i].fd = sd ;
            break ;
        }
    }
    if( i == OPEN_MAX ) {
        ostringstream os ;
        os<<"New connection request from client rejected : " ;
        char buf[50] ;
        string clientAddr ;
        if( inet_ntop( AF_INET, &client.sin_addr, buf, sizeof( buf ) ) != NULL ) {
            os<<buf<<":"<<ntohs(client.sin_port) ;
        }
        os<<endl<<"Maximum Listening capacity Reached : " <<OPEN_MAX<<endl ;
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, "void TcpLogServerImpl::acceptNewConnection_()", os.str() ) ;
    }

    sockets_[i].events = POLLIN ;
    if( i > highestIndex_ ) {
        highestIndex_ = i;
    }
    sdFptrRelations_.insert( map<int,int>::value_type( sd, UNDECIDED ) ) ;
    guard.noClose() ;
}

void TcpLogServerImpl::removeConnection_( int sd ) {
    
    for( int i = 1 ; i < OPEN_MAX ; i++ ) {
        if( sockets_[i].fd == sd ) {
            sockets_[i].fd = -1 ;
            break ;
        }
    }
    sdFptrRelations_.erase( sd ) ;
    close( sd ) ;
}

void TcpLogServerImpl::decideConnType_( int sd, const string &msg ) throw ( Exception ) {

    string logType = msg.substr( 0, 1 ) ;
    TcpLogServerImplFunPtrIndices l ;
    if( logType == string( 1,  ConnectionTypes::CRITICAL ) ) {
        l = CRITICAL ;
        }
    else if( logType == string( 1,  ConnectionTypes::ERROR ) ) {
        l = ERROR ;
        }
    else if( logType == string( 1,  ConnectionTypes::WARNING ) ) {
        l = WARNING ;
        }
    else if( logType == string( 1,  ConnectionTypes::AUDIT ) ) {
        l = AUDIT ;
        }
    else if( logType == string( 1,  ConnectionTypes::DEBUG ) ) {
        l = DEBUG ;
        }
    else if( logType == string( 1,  ConnectionTypes::FUNCALL ) ) {
        l = FUNCT_CALL ;
        }
    else {
        ostringstream os ;
        os<<"Message with unknow log type request received : " <<logType ;
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, "void TcpLogServerImpl::acceptNewConnection_()", os.str() ) ;
    }
    sdFptrRelations_[sd] = l ;
    if( msg.length() > 1 ) {
        (this->*logFunctPtrs_[l])( sd, msg.substr( 1 ) ) ;
    }
}

void TcpLogServerImpl::processClientMessage_( unsigned int numRdRdySocs ) throw ( Exception ) {

    if( numRdRdySocs <= 0 ) {
        return ;
    }

    int sd ;
    char buf[5000] ;
    int numBytesRd ;
    ostringstream os ;
    bool anyproblems = false ;
    for( int i = 1 ; i <= highestIndex_ ; i++ ) {
        if( ( sd = sockets_[i].fd ) < 0 ) {
            continue ;
        }
        if( sockets_[i].revents & ( POLLIN | POLLERR ) ) {
            if( ( numBytesRd = read( sd, buf, sizeof( buf ) ) ) < 0 ) {
                if( errno == ECONNRESET ) {
                    close( sd ) ;
                    sockets_[i].fd = -1 ;
                }
                else {
                    anyproblems = true ;
                    os<<"Read error for socket descriptor "<<sd<<" at index "<<i<<" of the fdset of poll()"<<endl ;
                }
            }
            else if( numBytesRd == 0 ) {
                removeConnection_( sd ) ;
            }
            else {
                buf[numBytesRd] = '\0' ;
                logMessage_( sd, buf ) ;
            }
            if ( --numRdRdySocs <= 0 ) {
                break ;
            }
        }
    }
    if( anyproblems ) {
        THROW_EXCEPTN_WWHERE( TcpLogServerImplException, \
            "void TcpLogServerImpl::processClientMessage_( unsigned int numRdRdySocs )", os.str() ) ;
    }
}

void TcpLogServerImpl::logMessage_( int sd, const string &msg ) throw ( ) {
    try {
        if( sdFptrRelations_.end() == sdFptrRelations_.find( sd ) ) {
            ostringstream os ;
            os<<"FATAL : Unknown socket descriptor received for logging : "<<sd<<endl ;
            os<<"Possiblly corrupt processing function pointer table" ;
            THROW_EXCEPTN_WWHERE( TcpLogServerImplException, \
                "void TcpLogServerImpl::logMessage_( int sd, const string &msg )", os.str() ) ;
        }
        int funptrIndex = sdFptrRelations_[sd] ;
        (this->*logFunctPtrs_[funptrIndex])( sd, msg ) ;
    }
    catch( Exception &e ) {
        string reason = e.reason() ;
        int i = reason.find( ' ' ) ;

        if( reason.substr( 0, i ) == "FATAL" ) {
            throw ;
        }
        writeLogServerMsgs( e.toString() ) ;
    }
    catch( ... ) {
        writeLogServerMsgs( "Unknown Exception Caught in TcpLogServerImpl::logMessage_()" ) ;
    }
}

void TcpLogServerImpl::run( BlockingOperationSentry* callbackObj ) throw ( Exception ) {

    callbackObj_ = callbackObj ;
    unsigned int numRdRdySocs ;
    while( 1 ) {

        try {
            bool newconnection = wait4ClientAction_( numRdRdySocs ) ;
            if( newconnection ) {
                acceptNewConnection_() ;
            }
            if( numRdRdySocs > 0 ) {
                processClientMessage_( numRdRdySocs ) ;
            }
        }
        catch( Exception &e ) {

            string reason = e.reason() ;
            int i = reason.find( ' ' ) ;

            if( reason.substr( 0, i ) == "FATAL" ) {
                throw ;
            }
            writeLogServerMsgs( e.toString() ) ;
        }
        catch( ... ) {
            writeLogServerMsgs( "Unknown Exception Caught in TcpLogServerImpl::run()" ) ;
        }
    }
}
