
//    Date : Sun Sep  1 20:09:28 EDT 2002
//    Author : Sony E Antony


//    File : ELF_TcpLogServerImpl.h
//    File belongs to ELF : Exception Handling and Logging Framework.
//    sonyeantony@yahoo.com


#ifndef __ELF_TCP_LOG_SERVER_IMPL_H__
#define __ELF_TCP_LOG_SERVER_IMPL_H__

#ifndef OPEN_MAX
#define OPEN_MAX    256
#endif

#include<limits.h>
#include <sys/poll.h>

#include "ELF_LogServer.h"
#include "ELF_LogWriter.h"
#include "ELF_Exception.h"
#include "ELF_UdpMessage.h"

namespace ELF {

DEFINE_EXCEPTION( TcpLogServerImplException ) ;

class TcpLogServerImpl : public LogServer {

    struct pollfd sockets_[OPEN_MAX] ;
    int listenSd_ ;
    unsigned int highestIndex_ ;
    map<int, int> sdFptrRelations_ ;
    void (TcpLogServerImpl::*logFunctPtrs_[7])( int sd, const string &msg ) ;

    int port_ ;
    int socket_ ;
    string logsrvrLogFileName_ ;
    BlockingOperationSentry* callbackObj_ ;
  public :

    TcpLogServerImpl() throw ( Exception ) ;
    ~TcpLogServerImpl() ;
    void run( BlockingOperationSentry* co = NULL ) throw ( Exception ) ;

  private :

    void logMessage_( int sd, const string &msg ) throw ( ) ;
    void processClientMessage_( unsigned int numRdRdySocs ) throw ( Exception ) ;
    void acceptNewConnection_() throw ( Exception ) ;
    bool wait4ClientAction_( unsigned int &numRdRdySocs ) throw ( Exception ) ;
    void setupSocket_() throw ( Exception ) ;
    void addNewConnection_( struct sockaddr_in &client, int sd ) throw ( Exception ) ;
    void removeConnection_( int sd ) ;

    void decideConnType_( int sd, const string &msg ) throw ( Exception ) ;
    void logCritical_( int sd, const string &msg ) {
        logWriter_->logCritical( msg ) ;
    }
    void logError_( int sd, const string &msg ) {
        logWriter_->logError( msg ) ;
    }
    void logWarning_( int sd, const string &msg ) {
        logWriter_->logWarning( msg ) ;
    }
    void logAudit_( int sd, const string &msg ) {
        logWriter_->logAudit( msg ) ;
    }
    void logDebug_( int sd, const string &msg ) {
        logWriter_->logDebug( msg ) ;
    }
    void logFunctCall_( int sd, const string &msg ) {
        logWriter_->logFunctCall( msg ) ;
    }
};

}
#endif
