/* From Meyers and Plum. "The New C: Low-Level I/O for Embedded Systems,"
 * C/C++ Users Journal, November 2003.
 *
 * This file declares the "functions" that operate on I/O Register designators
 * and I/O group designators specified in:
 * ISO/IEC TR 18037, C Extensions to Support Embedded Processors.
 *
 * Note: All names beginning with _ are private implementation details
 * only to be used by iohw.h, iohw.c, iolpt.h, and iolpt.c.
 *
 */

#ifndef _IOHW_H
#define _IOHW_H

/* Support four types of device registers: I/O port of 8 bits, I/O port of
 * 16 bits, memory-mapped 8 bit location, memory-mapped 16 bit location.  */
enum _Access_type {_Port8, _Port16, _Mem8, _Mem16};

/* Our private type for a I/O register designator is just a code telling
 * the type of device register and a union for the register address or
 * port number.  */
typedef struct _IOreg {
    enum _Access_type type;
    union {
	unsigned short port;
	volatile unsigned char *ptr8;
	volatile unsigned short *ptr16;
    } location;
} _IOreg;

typedef int _IOgroup;

typedef int ioindex_t;

extern void iogroup_acquire(_IOgroup group);
extern void iogroup_release(_IOgroup group);
extern void iogroup_map(_IOgroup dst_group, _IOgroup src_group);
extern unsigned int iord(_IOreg reg);
extern unsigned long iordl(_IOreg reg);
extern unsigned int iordbuf(_IOreg reg, ioindex_t ix);
extern unsigned long iordbufl(_IOreg reg, ioindex_t ix);
extern void iowr(_IOreg reg, unsigned int value);
extern void iowrl(_IOreg reg, unsigned long value);
extern void iowrbuf(_IOreg reg, ioindex_t ix, unsigned int value);
extern void iowrbufl(_IOreg reg, ioindex_t ix, unsigned long value);
extern void ioand(_IOreg reg, unsigned int value);
extern void ioor(_IOreg reg, unsigned int value);
extern void ioxor(_IOreg reg, unsigned int value);
extern void ioorl(_IOreg reg, unsigned long value);
extern void ioandl(_IOreg reg, unsigned long value);
extern void ioxorl(_IOreg reg, unsigned long value);
extern void ioandbuf(_IOreg reg, ioindex_t ix, unsigned int value);
extern void ioorbuf(_IOreg reg, ioindex_t ix, unsigned int value);
extern void ioxorbuf(_IOreg reg, ioindex_t ix, unsigned int value);
extern void ioandbufl(_IOreg reg, ioindex_t ix, unsigned long value);
extern void ioorbufl(_IOreg reg, ioindex_t ix, unsigned long value);
extern void ioxorbufl(_IOreg reg, ioindex_t ix, unsigned long value);

#endif /* _IOHW_H */
