// This is a minimal example of the use of template class Euler.
// dS.cpp is a fully fleshed-out program which integrates
//	the same differential equations.

#include <complex>
#include <valarray>
#include <integrators.h>
using namespace std;

namespace discretization_data
{
    const complex<double> i(0,1);
    unsigned int nmesh = 10;
}

// This function computes the right-hand sides of the differential equations.
// This will be return_derivs in the Integrator code.
void discretized_odes(const valarray<complex<double> > &psi,
                      const double t,
              valarray<complex<double> > &dpsidt)
{
    using namespace discretization_data;
    dpsidt[0] = i*(psi[1]-2.0*psi[0]);
    dpsidt[nmesh-1] = i*(psi[nmesh-2]-2.0*psi[nmesh-1]);
    for (int loop=1; loop<nmesh-1; loop++)
        dpsidt[loop] = i*(psi[loop+1] - 2.0*psi[loop]
                      + psi[loop-1]);
}

int main()
{
  using namespace discretization_data;
  valarray<complex<double> > psi(complex<double>(1.0/nmesh),nmesh);

  Euler< valarray<complex<double> >, double, complex<double> >
    dSchrodinger(psi, 0.0, 0.01, discretized_odes );

  while (dSchrodinger.current_time() < 50)
      dSchrodinger.step();
  // Some output statements would be added to a real program...
}
