///////////////////////////////////////////////////////////////////////////////////////////////
// Filename:    SDESolverWin32.h
// Package:     SDESolver
// Purpose:     Simple usage demonstration. Create a console application and add all .h and
//              .cpp files except for SDESolver.cpp and DynSystemModule.*
//              Simulates a stochastic elliptic limit cycle system
// Written:     Dave Burke        03/08/02
///////////////////////////////////////////////////////////////////////////////////////////////

#include "../SolverFactory.h"
#include "../Solver.h"
#include "../SolverSettings.h"
#include "../EllipseDynSystem.h"

int main(int argc, char* argv[])
{
    // Settings
    CSolverSettings settings;
    settings.m_strOutputFileName = "output.dat";
    settings.m_ICs.m_pData = (float*)malloc(sizeof(float) * 2);
    settings.m_ICs.m_nDim = 2;
    settings.m_ICs.m_pData[0] = 0.1;
    settings.m_ICs.m_pData[1] = 0.1;
    settings.m_nT = 100;
    settings.m_nStepSize = .01;
    settings.m_nMultiplier = 1;
    settings.m_strWienerOutputFileName = "wiener.dat";
    settings.m_nSeed = 101;

    // Create solver
    CSolver* pSolver = CSolverFactory::CreateSolver(CSolverFactory::RUNGE_KUTTA, settings);
    if (NULL == pSolver)
    {
        fprintf(stderr, "Could not create solver\n");
        return 1;
    }

    // Register a system
    CDynSystem* pSystem = new CEllipseDynSystem();
    pSolver->RegisterDynSystem(pSystem);

    // Run!
    if (!pSolver->Run())
    {
        fprintf(stderr, "%s\n", pSolver->GetLastError().c_str());
        delete pSolver;
        return 1;
    }

    // Cleanup
    delete pSolver;
    delete pSystem;

    return 0;
}
